import "Turbine.UI";
import "Turbine.UI.Lotro";
import "PengorosPlugins.Utils";
import "PengorosPlugins.BuffBars";
-- import the dragbar control if it is available
local importDragBar = function()
    import "Deusdictum.UI";
end
hasDragBar = pcall(importDragBar);

-- load the plugin pluginSettings
pluginSettings = Settings();

-- plugin windows
local effectWindows = {};
local effectSliders = {};
local quickslotBars = {};
local effectLogger = nil;

-- reset windows
function ResetState()
    local i;
    -- destroy the existing effect windows
    for i = 1, table.getn(effectWindows) do
        effectWindows[i]:Destroy();
        effectWindows[i] = nil;
    end
    effectWindows = nil;
    
    -- destroy the existing effect sliders
    for i = 1, table.getn(effectSliders) do
        effectSliders[i]:Destroy();
        effectSliders[i] = nil;
    end
    effectSliders = nil;

    -- destroy existing quick slots
    for i = 1, table.getn(quickslotBars) do
        quickslotBars[i]:Destroy();
        quickslotBars[i] = nil;
    end
    quickslotBars = nil;
    
    -- destroy the existing logger
    if (effectLogger ~= nil) then
        effectLogger:Destroy();
        effectLogger = nil;
    end

    -- create new effect windows    
    effectWindows = {};
    for i = 1, table.getn(pluginSettings.windows) do
        if (pluginSettings.windows[i].enabled) then
            table.insert(effectWindows, EffectWindow(pluginSettings.windows[i]));
        end
    end

    -- create new effect sliders    
    effectSliders = {};
    for i = 1, table.getn(pluginSettings.sliders) do
        if (pluginSettings.sliders[i].enabled) then
            table.insert(effectSliders, SliderWindow(pluginSettings.sliders[i]));
        end
    end

    -- create new quickslot bars    
    quickslotBars = {};
    for i = 1, table.getn(pluginSettings.quickslots) do
        if (pluginSettings.quickslots[i].enabled) then
            table.insert(quickslotBars, QuickslotBar(pluginSettings.quickslots[i]));
        end
    end
    
    -- effect logger
    if (pluginSettings.log.enabled) then
        effectLogger = Log(pluginSettings.log);
    end
end

-- logging
function EnableLogging(enable)
    if (pluginSettings.log.enabled ~= enable) then
        pluginSettings.log.enabled = enable;
        if (enable) then
            Turbine.Shell.WriteLine(L["Effect Logging Enabled"]);
            effectLogger = Log(pluginSettings.log);
        else
            Turbine.Shell.WriteLine(L["Effect Logging Disabled"]);
            effectLogger:Destroy();
            effectLogger = nil;
        end
    end
end

-- commands
buffBarsCommand = Turbine.ShellCommand();
Turbine.Shell.AddCommand("buffbars", buffBarsCommand);
function buffBarsCommand:GetShortHelp()
    return L["usage: /buffbars options | logging <enable|disable>"];
end
function buffBarsCommand:Execute(cmd, args)
    if (args == L["options"] or args == L["setup"]) then
        OptionsWindow.ShowOptions();
    elseif (args == L["logging enable"]) then
        EnableLogging(true);
    elseif (args == L["logging disable"]) then
        EnableLogging(false);
    else
        Turbine.Shell.WriteLine(self:GetShortHelp());
    end
end

-- plugin unload event
function UnloadPlugin()
    local i;
    for i = 1, table.getn(effectWindows) do
        effectWindows[i]:Destroy();
        effectWindows[i] = nil;
    end
    for i = 1, table.getn(effectSliders) do
        effectSliders[i]:Destroy();
        effectSliders[i] = nil;
    end
    for i = 1, table.getn(quickslotBars) do
        quickslotBars[i]:Destroy();
        quickslotBars[i] = nil;
    end
    if (effectLogger ~= nil) then
        effectLogger:Destroy();
        effectLogger = nil;
    end
    if (OptionsWindow.instance ~= nil) then
        OptionsWindow.instance:Destroy();
    end
    pluginSettings:Save();
end

-- register the unload
PengorosPlugins.Utils.KeyManager.instance:RegisterUnloadEvent("BuffBars", UnloadPlugin);

-- set the initial state
ResetState();

Turbine.Shell.WriteLine("BuffBars v1.1.1 by Pengoros");
