import "Turbine.Gameplay";
import "PengorosPlugins.BuffBars";

Settings = class();

-- enums
Settings.UndefinedEffect = Turbine.Gameplay.EffectCategory.Undefined;
Settings.DiseaseEffect = Turbine.Gameplay.EffectCategory.Disease;
Settings.FearEffect = Turbine.Gameplay.EffectCategory.Fear;
Settings.PoisonEffect = Turbine.Gameplay.EffectCategory.Poison;
Settings.WoundEffect = Turbine.Gameplay.EffectCategory.Wound;
Settings.PhysicalEffect = Turbine.Gameplay.EffectCategory.Physical;
Settings.ElementalEffect = Turbine.Gameplay.EffectCategory.Elemental;
Settings.SongEffect = Turbine.Gameplay.EffectCategory.Song;
Settings.CorruptionEffect = Turbine.Gameplay.EffectCategory.Corruption;
Settings.CryEffect = Turbine.Gameplay.EffectCategory.Cry;
Settings.Flash = "flash";

function Settings:Constructor()
    self:Load();
end

function Settings:Load()
    local saved = Turbine.PluginData.Load(Turbine.DataScope.Character, "BuffBarsSettings");

    if (type(saved) ~= "table") then
        -- set some reasonable defaults
        self:SetDefaults();
        self:Save();
    else
        -- copy the values into this object
        for i,v in pairs(saved) do
            if (i == "windows") then
                self.windows = {};
                local size = table.getn(v);
                for i = 1, size do
                    self.windows[i] = EffectBarSettings(v[i]);
                end
            elseif (i == "quickslots") then
                self.quickslots = {};
                local size = table.getn(v);
                for i = 1, size do
                    self.quickslots[i] = QuickslotSettings(v[i]);
                end
            elseif (i == "sliders") then
                self.sliders = {};
                local size = table.getn(v);
                for i = 1, size do
                    self.sliders[i] = SliderSettings(v[i]);
                end
            else
                self[i] = v;
            end
        end
        
        -- see if we need to do a migration
        self:CheckMigration();
    end
end

function Settings:Save()
    Turbine.PluginData.Save(Turbine.DataScope.Character, "BuffBarsSettings", self);
end

function Settings:Migrate(version)
    Turbine.Shell.WriteLine("Migrating BuffBars options to version " .. version);
    self.version = version;
end

function Settings:CheckMigration()
    local save = false;
    if (self.version == 1) then
        self.fillBottomToTop = false;
        self.showEffectName = true;
        self.showEffectTimer = true;
        self.timerWidth = nil;
        self:Migrate(2);
        save = true;
    end
    if (self.version == 2) then
        self.quickslots = {QuickslotSettings()};
        self.quickslots[1].enabled = self.showQuickslot;
        self.showQuickslot = nil;
        self.quickslots[1].qsX = self.qsX;
        self.qsX = nil;
        self.quickslots[1].qsY = self.qsY;
        self.qsY = nil;
        self.quickslots[1].moralePercent = self.moralePercent;
        self.moralePercent = nil;
        self.quickslots[1].powerPercent = self.powerPercent;
        self.powerPercent = nil;
        self.quickslots[1].moraleType = self.moraleType;
	    self.moraleType = nil;
        self.quickslots[1].moraleData = self.moraleData;
	    self.moraleData = nil;
        self.quickslots[1].powerType = self.powerType;
	    self.powerType = nil;
        self.quickslots[1].powerData = self.powerData;
	    self.powerData = nil;
        self.quickslots[1].diseaseType = self.diseaseType;
	    self.diseaseType = nil;
        self.quickslots[1].diseaseData = self.diseaseData;
	    self.diseaseData = nil;
        self.quickslots[1].fearType = self.fearType;
	    self.fearType = nil;
        self.quickslots[1].fearData = self.fearData;
	    self.fearData = nil;
        self.quickslots[1].poisonType = self.poisonType;
        self.poisonType = nil;
        self.quickslots[1].poisonData = self.poisonData;
        self.poisonData = nil;
        self.quickslots[1].woundType = self.woundType;
        self.woundType = nil;
        self.quickslots[1].woundData = self.woundData;
        self.woundData = nil;
        
        self.windows = {EffectBarSettings()};
        self.windows[1].posX = self.posX;
    	self.posX = nil;
        self.windows[1].posY = self.posY;
    	self.posY = nil;
        self.windows[1].barHeight = self.barHeight;
    	self.barHeight = nil;
        self.windows[1].barWidth = self.barWidth;
    	self.barWidth = nil;
        self.windows[1].showDebuffBar = self.showDebuffBar;
    	self.showDebuffBar = nil;
        self.windows[1].showBuffBar = self.showBuffBar;
    	self.showBuffBar = nil;
        self.windows[1].showDisease = self.showDisease;
    	self.showDisease = nil;
        self.windows[1].showFear = self.showFear;
    	self.showFear = nil;
        self.windows[1].showPoison = self.showPoison;
    	self.showPoison = nil;
        self.windows[1].showWound = self.showWound;
    	self.showWound = nil;
        self.windows[1].showCorruption = self.showCorruption;
    	self.showCorruption = nil;
        self.windows[1].showCry = self.showCry;
    	self.showCry = nil;
        self.windows[1].showElemental = self.showElemental;
    	self.showElemental = nil;
        self.windows[1].showPhysical = self.showPhysical;
    	self.showPhysical = nil;
        self.windows[1].showSong = self.showSong;
    	self.showSong = nil;
        self.windows[1].showUndefined = self.showUndefined;
    	self.showUndefined = nil;
        self.windows[1].showCurableOnly = self.showCurableOnly;
    	self.showCurableOnly = nil;
        self.windows[1].ignored = self.ignored;
    	self.ignored = nil;
        self.windows[1].showEffectName = self.showEffectName;
    	self.showEffectName = nil;
        self.windows[1].showEffectTimer = self.showEffectTimer;
    	self.showEffectTimer = nil;
        self.windows[1].backgroundColor = self.backgroundColor;
    	self.backgroundColor = nil;
        self.windows[1].font = self.font;
    	self.font = nil;
        self.windows[1].sortMethod = self.sortMethod;
    	self.sortMethod = nil;
        self.windows[1].sortAscending = self.sortAscending;
    	self.sortAscending = nil;
        self.windows[1].showDebuffsFirst = self.showDebuffsFirst;
    	self.showDebuffsFirst = nil;
        self.windows[1].fillBottomToTop = self.fillBottomToTop;
    	self.fillBottomToTop = nil;
        self.windows[1].allowFlashing = self.allowFlashing;
    	self.allowFlashing = nil;
        self.windows[1].flashTime = self.flashTime;
    	self.flashTime = nil;

        local col = Settings.Flash;
        self.windows[1].colors[col].remaining = self.flashRemainingColor;
        self.windows[1].colors[col].elapsed = self.flashElapsedColor;
        self.flashRemainingColor = nil;
        self.flashElapsedColor = nil;
        
        col = Settings.UndefinedEffect;
        self.windows[1].colors[col].remaining = self.undefinedRemainingColor;
        self.windows[1].colors[col].elapsed = self.undefinedElapsedColor;
        self.undefinedRemainingColor = nil;
        self.undefinedElapsedColor = nil;
        
        col = Settings.DiseaseEffect;
        self.windows[1].colors[col].remaining = self.diseaseRemainingColor;
        self.windows[1].colors[col].elapsed = self.diseaseElapsedColor;
        self.diseaseRemainingColor = nil;
        self.diseaseElapsedColor = nil;
        
        col = Settings.FearEffect;
        self.windows[1].colors[col].remaining = self.fearRemainingColor;
        self.windows[1].colors[col].elapsed = self.fearElapsedColor;
        self.fearRemainingColor = nil;
        self.fearElapsedColor = nil;
    
        col = Settings.PoisonEffect;
        self.windows[1].colors[col].remaining = self.poisonRemainingColor;
        self.windows[1].colors[col].elapsed = self.poisonElapsedColor;
        self.poisonRemainingColor = nil;
        self.poisonElapsedColor = nil;
    
        col = Settings.WoundEffect;
        self.windows[1].colors[col].remaining = self.woundRemainingColor;
        self.windows[1].colors[col].elapsed = self.woundElapsedColor;
        self.woundRemainingColor = nil;
        self.woundElapsedColor = nil;
    
        col = Settings.PhysicalEffect;
        self.windows[1].colors[col].remaining = self.physicalRemainingColor;
        self.windows[1].colors[col].elapsed = self.physicalElapsedColor;
        self.physicalRemainingColor = nil;
        self.physicalElapsedColor = nil;
    
        col = Settings.ElementalEffect;
        self.windows[1].colors[col].remaining = self.elementalRemainingColor;
        self.windows[1].colors[col].elapsed = self.elementalElapsedColor;
        self.elementalRemainingColor = nil;
        self.elementalElapsedColor = nil;
    
        col = Settings.SongEffect;
        self.windows[1].colors[col].remaining = self.songRemainingColor;
        self.windows[1].colors[col].elapsed = self.songElapsedColor;
        self.songRemainingColor = nil;
        self.songElapsedColor = nil;
    
        col = Settings.CorruptionEffect;
        self.windows[1].colors[col].remaining = self.corruptionRemainingColor;
        self.windows[1].colors[col].elapsed = self.corruptionElapsedColor;
        self.corruptionRemainingColor = nil;
        self.corruptionElapsedColor = nil;
    
        col = Settings.CryEffect;
        self.windows[1].colors[col].remaining = self.cryRemainingColor;
        self.windows[1].colors[col].elapsed = self.cryElapsedColor;
        self.cryRemainingColor = nil;
        self.cryElapsedColor = nil;
        
        self:Migrate(3);
        save = true;
    end
    if (self.version == 3) then
        self.log = LogSettings();
        local i;
        for i = 1, table.getn(self.windows) do
            self.windows[i].showDiseaseCD = true;
            self.windows[i].showFearCD = true;
            self.windows[i].showPoisonCD = true;
            self.windows[i].showWoundCD = true;
            self.windows[i].showMoraleCD = true;
            self.windows[i].showPowerCD = true;
            self.windows[i].showEffectDuration = true;
            self.windows[i].minDurationFilter = false;
            self.windows[i].minDuration = 5;
            self.windows[i].maxDurationFilter = false;
            self.windows[i].maxDuration = 300;
            self.windows[i].visibleType = self.windows[i].onlyInCombat and EffectWindow.VisibleInCombat or EffectWindow.VisibleAlways;
            self.windows[i].onlyInCombat = nil;
        end
        for i = 1, table.getn(self.quickslots) do
            self.quickslots[i].visibleType = self.quickslots[i].onlyInCombat and EffectWindow.VisibleInCombat or EffectWindow.VisibleAlways;
            self.quickslots[i].onlyInCombat = nil;
        end

        self:Migrate(4);
        save = true;
    end
    if (self.version == 4) then
        for i = 1, table.getn(self.windows) do
            if (not self.windows[i].minDurationFilter) then
                self.windows[i].minDuration = 0;
            end
            self.windows[i].minDurationFilter = nil;
            if (not self.windows[i].maxDurationFilter) then
                self.windows[i].maxDuration = 0;
            end
            self.windows[i].maxDurationFilter = nil;
            self.windows[i].displayFormat = EffectDisplayFormat.Clock;
        end
            
        self.sliders = {};
        self:Migrate(5);
        save = true;
    end
    
    if (save) then
        self:Save();
    end
end

function Settings:SetDefaults()
    -- current settings version
    self.version = 5;

    -- quickslot 
    self.quickslots = {QuickslotSettings()};
    
    -- effect windows
    self.windows = {EffectBarSettings()};
    
    -- effect sliders
    self.sliders = {SliderSettings()};
    
    -- logging
    self.log = LogSettings();
end
