import "Turbine.Gameplay";

EffectManager = class();

function EffectManager:Constructor()
    -- listeners
    self.listeners = {};
    
    -- effect hooks
    self.player = Turbine.Gameplay.LocalPlayer();
    local effects = self.player:GetEffects();
    
    effects.EffectAdded = function(sender, args)
        local effect = self.player:GetEffects():Get(args.Index);
        self:AddEffect(effect);
    end
    
    effects.EffectRemoved = function(sender, args)
        self:RemoveEffect(args.Effect);
    end
    
    effects.EffectsCleared = function(sender, args)
        self:ClearEffects();
    end
end

function EffectManager:AddListener(listener)
    -- make sure it isn't in the list already
    self:RemoveListener(listener);
    table.insert(self.listeners, listener);
end

function EffectManager:RemoveListener(listener)
    local i;
    for i = 1, table.getn(self.listeners) do
        if (self.listeners[i] == listener) then
            table.remove(self.listeners, i);
            break;
        end
    end
end

function EffectManager:GetEffects()
    return self.player:GetEffects();
end

function EffectManager:AddEffect(effect)
    local i;
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].EffectAdded) == "function") then
            self.listeners[i]:EffectAdded(effect);
        end
    end
end

function EffectManager:RemoveEffect(effect)
    local i;
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].EffectRemoved) == "function") then
            self.listeners[i]:EffectRemoved(effect);
        end
    end
end

function EffectManager:ClearEffects()
    local i;
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].EffectsCleared) == "function") then
            self.listeners[i]:EffectsCleared();
        end
    end
end

-- define a global instance
EffectManager.instance = EffectManager();
