import "Turbine.Gameplay";

PlayerManager = class();

function PlayerManager:Constructor()
    -- listeners
    self.listeners = {};
    
    self.player = Turbine.Gameplay.LocalPlayer();

    -- trap the player events
    self.player.InCombatChanged = function(sender, args)
        self:FireInCombat();
    end

    self.player.MoraleChanged = function(sender, args)
        self:FireMorale();
    end
    
    self.player.PowerChanged = function(sender, args)
        self:FirePower();
    end
end

function PlayerManager:GetPlayer()
    return self.player;
end

function PlayerManager:AddListener(listener)
    -- make sure it isn't in the list already
    self:RemoveListener(listener);
    table.insert(self.listeners, listener);
end

function PlayerManager:RemoveListener(listener)
    local i;
    for i = 1, table.getn(self.listeners) do
        if (self.listeners[i] == listener) then
            table.remove(self.listeners, i);
            break;
        end
    end
end

function PlayerManager:FireInCombat()
    local i;
    local inCombat = self.player:IsInCombat();
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].InCombatChanged) == "function") then
            self.listeners[i]:InCombatChanged(inCombat);
        end
    end
end

function PlayerManager:FireMorale()
    local i;
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].MoraleChanged) == "function") then
            self.listeners[i]:MoraleChanged(self.player);
        end
    end
end

function PlayerManager:FirePower()
    local i;
    for i = 1, table.getn(self.listeners) do
        if (type(self.listeners[i].PowerChanged) == "function") then
            self.listeners[i]:PowerChanged(self.player);
        end
    end
end

-- define a global instance
PlayerManager.instance = PlayerManager();
